"""
    Pod API

    This document refers to Symphony API calls that do not need encryption or decryption of content.  - sessionToken can be obtained by calling the authenticationAPI on the symphony back end and the key manager respectively. Refer to the methods described in authenticatorAPI.yaml. - Actions are defined to be atomic, ie will succeed in their entirety or fail and have changed nothing. - If it returns a 40X status then it will have made no change to the system even if ome subset of the request would have succeeded. - If this contract cannot be met for any reason then this is an error and the response code will be 50X.   # noqa: E501

    The version of the OpenAPI document: 20.12.1
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from symphony.bdk.gen.api_client import ApiClient, Endpoint as _Endpoint
from symphony.bdk.gen.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from symphony.bdk.gen.pod_model.error import Error
from symphony.bdk.gen.pod_model.string_id import StringId
from symphony.bdk.gen.pod_model.success_response import SuccessResponse
from symphony.bdk.gen.pod_model.user_id_list import UserIdList
from symphony.bdk.gen.pod_model.v2_presence import V2Presence
from symphony.bdk.gen.pod_model.v2_presence_list import V2PresenceList
from symphony.bdk.gen.pod_model.v2_presence_status import V2PresenceStatus
from symphony.bdk.gen.pod_model.v2_user_presence import V2UserPresence


class PresenceApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __v1_presence_feed_create_post(
            self,
            session_token,
            **kwargs
        ):
            """Create Presence status feed.  # noqa: E501

            Create a new stream capturing presence status changes (\"presence feed\"). When read from, the feed will return the current presence status of company (pod) users if it has changed since the last read.  Returns the ID of the presence feed to be used in subsequent read operations.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_presence_feed_create_post(session_token, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                StringId
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v1_presence_feed_create_post = _Endpoint(
            settings={
                'response_type': (StringId,),
                'auth': [],
                'endpoint_path': '/v1/presence/feed/create',
                'operation_id': 'v1_presence_feed_create_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                ],
                'required': [
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_presence_feed_create_post
        )

        def __v1_presence_feed_feed_id_delete_post(
            self,
            session_token,
            feed_id,
            **kwargs
        ):
            """Delete a presence status feed.  # noqa: E501

            Returns the ID of the deleted feed.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_presence_feed_feed_id_delete_post(session_token, feed_id, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                feed_id (str): Presence feed ID.

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                StringId
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['feed_id'] = \
                feed_id
            return self.call_with_http_info(**kwargs)

        self.v1_presence_feed_feed_id_delete_post = _Endpoint(
            settings={
                'response_type': (StringId,),
                'auth': [],
                'endpoint_path': '/v1/presence/feed/{feedId}/delete',
                'operation_id': 'v1_presence_feed_feed_id_delete_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'feed_id',
                ],
                'required': [
                    'session_token',
                    'feed_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'feed_id':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'feed_id': 'feedId',
                },
                'location_map': {
                    'session_token': 'header',
                    'feed_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_presence_feed_feed_id_delete_post
        )

        def __v1_presence_feed_feed_id_read_get(
            self,
            session_token,
            feed_id,
            **kwargs
        ):
            """Read a presence status feed.  # noqa: E501

            Returns the current presence status of company (pod) users if it has changed since the last read. Returns up to 500 records at a time.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_presence_feed_feed_id_read_get(session_token, feed_id, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                feed_id (str): Presence feed ID.

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V2PresenceList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['feed_id'] = \
                feed_id
            return self.call_with_http_info(**kwargs)

        self.v1_presence_feed_feed_id_read_get = _Endpoint(
            settings={
                'response_type': (V2PresenceList,),
                'auth': [],
                'endpoint_path': '/v1/presence/feed/{feedId}/read',
                'operation_id': 'v1_presence_feed_feed_id_read_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'feed_id',
                ],
                'required': [
                    'session_token',
                    'feed_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'feed_id':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'feed_id': 'feedId',
                },
                'location_map': {
                    'session_token': 'header',
                    'feed_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_presence_feed_feed_id_read_get
        )

        def __v1_user_presence_register_post(
            self,
            session_token,
            uid_list,
            **kwargs
        ):
            """Register interest in a user's presence status  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_user_presence_register_post(session_token, uid_list, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                uid_list (UserIdList): List of (integer) User IDs of users whose presence to query

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                SuccessResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['uid_list'] = \
                uid_list
            return self.call_with_http_info(**kwargs)

        self.v1_user_presence_register_post = _Endpoint(
            settings={
                'response_type': (SuccessResponse,),
                'auth': [],
                'endpoint_path': '/v1/user/presence/register',
                'operation_id': 'v1_user_presence_register_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'uid_list',
                ],
                'required': [
                    'session_token',
                    'uid_list',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'uid_list':
                        (UserIdList,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                    'uid_list': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_user_presence_register_post
        )

        def __v2_user_presence_get(
            self,
            session_token,
            **kwargs
        ):
            """Get presence information about the requesting user.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v2_user_presence_get(session_token, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V2Presence
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v2_user_presence_get = _Endpoint(
            settings={
                'response_type': (V2Presence,),
                'auth': [],
                'endpoint_path': '/v2/user/presence',
                'operation_id': 'v2_user_presence_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                ],
                'required': [
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v2_user_presence_get
        )

        def __v2_user_presence_post(
            self,
            session_token,
            presence,
            **kwargs
        ):
            """Set the presence of the requesting user.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v2_user_presence_post(session_token, presence, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                presence (V2PresenceStatus):

            Keyword Args:
                soft (bool): If true, the user's current status is taken into consideration. If the user is currently OFFLINE, the user's presence will still be OFFLINE, but the new presence will take effect when the user comes online. If the user is currently online, the user's activity state will be applied to the presence if applicable. (e.g. if you are setting their presence to AVAILABLE, but the user is currently idle, their status will be represented as AWAY) . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V2Presence
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['presence'] = \
                presence
            return self.call_with_http_info(**kwargs)

        self.v2_user_presence_post = _Endpoint(
            settings={
                'response_type': (V2Presence,),
                'auth': [],
                'endpoint_path': '/v2/user/presence',
                'operation_id': 'v2_user_presence_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'presence',
                    'soft',
                ],
                'required': [
                    'session_token',
                    'presence',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'presence':
                        (V2PresenceStatus,),
                    'soft':
                        (bool,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'soft': 'soft',
                },
                'location_map': {
                    'session_token': 'header',
                    'presence': 'body',
                    'soft': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v2_user_presence_post
        )

        def __v2_users_presence_get(
            self,
            session_token,
            **kwargs
        ):
            """Get presence information about all company (pod) users.  # noqa: E501

            The returned data is taken from the in-memory cache for performance reasons which means inactive users may be omitted from the response.  All non-inactive users WILL be returned and some inactive users MAY be included. Any omitted user IS inactive.  Returned records are sorted by user ID, ascending.  This method is expensive. It pulls ALL records from the cache, sorts them and then only uses a subset. For large numbers of users, this can be very inefficient both due to sorting and due to the cache being distributed across many nodes.  Addiionally, there is the potential to miss users if they become active after the page in which their user ID falls has already been read by the client. To avoid this situation, a presence feed should be created (and optionally read from) first to capture presence changes of users who get reactivated during a paged call to this endpoint.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v2_users_presence_get(session_token, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.

            Keyword Args:
                last_user_id (int): Last user ID retrieved. Used for paging; if provided, results will skip users with IDs less than this parameter.. [optional]
                limit (int): Max number of records to return. If no value is provided, 1000 is the default. The maximum supported value is 5000.. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V2PresenceList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v2_users_presence_get = _Endpoint(
            settings={
                'response_type': (V2PresenceList,),
                'auth': [],
                'endpoint_path': '/v2/users/presence',
                'operation_id': 'v2_users_presence_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'last_user_id',
                    'limit',
                ],
                'required': [
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'last_user_id':
                        (int,),
                    'limit':
                        (int,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'last_user_id': 'lastUserId',
                    'limit': 'limit',
                },
                'location_map': {
                    'session_token': 'header',
                    'last_user_id': 'query',
                    'limit': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v2_users_presence_get
        )

        def __v3_user_presence_post(
            self,
            session_token,
            presence,
            **kwargs
        ):
            """Set presence information for a particular user.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v3_user_presence_post(session_token, presence, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                presence (V2UserPresence):

            Keyword Args:
                soft (bool): If true, the user's current status is taken into consideration. If the user is currently OFFLINE, the user's presence will still be OFFLINE, but the new presence will take effect when the user comes online. If the user is currently online, the user's activity state will be applied to the presence if applicable. (e.g. if you are setting their presence to AVAILABLE, but the user is currently idle, their status will be represented as AWAY) . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V2Presence
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['presence'] = \
                presence
            return self.call_with_http_info(**kwargs)

        self.v3_user_presence_post = _Endpoint(
            settings={
                'response_type': (V2Presence,),
                'auth': [],
                'endpoint_path': '/v3/user/presence',
                'operation_id': 'v3_user_presence_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'presence',
                    'soft',
                ],
                'required': [
                    'session_token',
                    'presence',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'presence':
                        (V2UserPresence,),
                    'soft':
                        (bool,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'soft': 'soft',
                },
                'location_map': {
                    'session_token': 'header',
                    'presence': 'body',
                    'soft': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v3_user_presence_post
        )

        def __v3_user_uid_presence_get(
            self,
            uid,
            session_token,
            **kwargs
        ):
            """Get presence information about a particular user.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v3_user_uid_presence_get(uid, session_token, async_req=True)
            >>> result = thread.get()

            Args:
                uid (int): User ID as a decimal integer 
                session_token (str): Session authentication token.

            Keyword Args:
                local (bool): If true, a local query will be performed and the presence will be set to OFFLINE for users who are not local to the calling user's pod. If false or absent, then the presence of all local users and the presence of all external users to whom the calling user is connected will be queried.  For external users, a \"presence interest\" should be registered through /v1/user/presence/register before querying for presence. . [optional] if omitted the server will use the default value of False
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V2Presence
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['uid'] = \
                uid
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v3_user_uid_presence_get = _Endpoint(
            settings={
                'response_type': (V2Presence,),
                'auth': [],
                'endpoint_path': '/v3/user/{uid}/presence',
                'operation_id': 'v3_user_uid_presence_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'uid',
                    'session_token',
                    'local',
                ],
                'required': [
                    'uid',
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'uid':
                        (int,),
                    'session_token':
                        (str,),
                    'local':
                        (bool,),
                },
                'attribute_map': {
                    'uid': 'uid',
                    'session_token': 'sessionToken',
                    'local': 'local',
                },
                'location_map': {
                    'uid': 'path',
                    'session_token': 'header',
                    'local': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v3_user_uid_presence_get
        )



from symphony.bdk.core.auth.auth_session import AuthSession
from symphony.bdk.core.config.model.bdk_retry_config import BdkRetryConfig
from symphony.bdk.core.retry import retry
from symphony.bdk.gen.pod_api.app_entitlement_api import AppEntitlementApi
from symphony.bdk.gen.pod_api.application_api import ApplicationApi
from symphony.bdk.gen.pod_model.application_detail import ApplicationDetail
from symphony.bdk.gen.pod_model.pod_app_entitlement import PodAppEntitlement
from symphony.bdk.gen.pod_model.pod_app_entitlement_list import PodAppEntitlementList
from symphony.bdk.gen.pod_model.user_app_entitlement import UserAppEntitlement
from symphony.bdk.gen.pod_model.user_app_entitlement_list import UserAppEntitlementList


class ApplicationService:
    """Service class for managing the applications and the application entitlements.

    This services used for retrieving information about a particular application or application entitlements,
    performing some actions related to the applications like:

    * Create an application
    * Update an existing application
    * Delete an existing application
    * Get the information of an existing application
    * Update application entitlements
    * Update user applications
    """

    def __init__(self, application_api: ApplicationApi, app_entitlement_api: AppEntitlementApi,
                 auth_session: AuthSession, retry_config: BdkRetryConfig):
        self._application_api = application_api
        self._app_entitlement_api = app_entitlement_api
        self._auth_session = auth_session
        self._retry_config = retry_config

    @retry
    async def create_application(self, application_detail: ApplicationDetail) -> ApplicationDetail:
        """
        Create a new application.

        See:

        * `Create Application <https://developers.symphony.com/restapi/reference#create-app>`_
        * `Create Application with an RSA Public Key <https://developers.symphony.com/restapi/reference#create-application-with-an-rsa-public-key>`_

        :param application_detail:  Contains the following fields for creating an application: appId, name, appUrl,
                                    domain, and publisher. Note that appUrl is not required.

        :return: The created application.

        """
        params = {
            'application_detail': application_detail,
            'session_token': await self._auth_session.session_token
        }
        return await self._application_api.v1_admin_app_create_post(**params)

    @retry
    async def update_application(self, app_id: str, application_detail: ApplicationDetail) -> ApplicationDetail:
        """
        Update an existing application.

        See:

        * `Update Application <https://developers.symphony.com/restapi/reference#update-application>`_
        * `Update Application with an RSA Public Key <https://developers.symphony.com/restapi/reference#update-application-with-an-rsa-public-key>`_

        :param app_id:              Id of the application needs to be updated.
        :param application_detail:  Contains the following fields for creating an application: appId, name, appUrl,
                                    domain, and publisher. Note that appUrl is not required.

        :return: The updated application.

        """
        params = {
            'id': app_id,
            'application_detail': application_detail,
            'session_token': await self._auth_session.session_token
        }
        return await self._application_api.v1_admin_app_id_update_post(**params)

    @retry
    async def delete_application(self, app_id: str) -> None:
        """
        Delete an existing application.

        See: `Delete Application <https://developers.symphony.com/restapi/reference#delete-application>`_

        :param app_id:  Id of the application needs to be deleted.

        """
        params = {
            'id': app_id,
            'session_token': await self._auth_session.session_token
        }
        await self._application_api.v1_admin_app_id_delete_post(**params)

    @retry
    async def get_application(self, app_id: str) -> ApplicationDetail:
        """
        Get an existing application.

        See: `Get Application <https://developers.symphony.com/restapi/reference#get-application>`_

        :param app_id:  Id of the application.

        :return: The detail of the lookup application.

        """
        params = {
            'id': app_id,
            'session_token': await self._auth_session.session_token
        }
        return await self._application_api.v1_admin_app_id_get_get(**params)

    @retry
    async def list_application_entitlements(self) -> [PodAppEntitlement]:
        """
        Get the list of application entitlements for the company.

        See: `List App Entitlements <https://developers.symphony.com/restapi/reference#list-app-entitlements>`_

        :return: The list of application entitlements.

        """
        params = {
            'session_token': await self._auth_session.session_token
        }
        pod_app_entitlement_list = await self._app_entitlement_api.v1_admin_app_entitlement_list_get(**params)
        return pod_app_entitlement_list.value

    @retry
    async def update_application_entitlements(self, entitlements: [PodAppEntitlement]) -> [PodAppEntitlement]:
        """
        Update the list of application entitlements for the company.

        See: `Update App Entitlements <https://developers.symphony.com/restapi/reference#update-application-entitlements>`_

        :param entitlements: The list of entitlements to be updated by.

        :return: The updated list of entitlements.

        """
        params = {
            'payload': PodAppEntitlementList(value=entitlements),
            'session_token': await self._auth_session.session_token
        }
        pod_app_entitlement_list = await self._app_entitlement_api.v1_admin_app_entitlement_list_post(**params)
        return pod_app_entitlement_list.value

    @retry
    async def list_user_applications(self, user_id: int) -> [UserAppEntitlement]:
        """
        Get the list of Symphony application entitlements for a particular user.

        See: `User Apps <https://developers.symphony.com/restapi/reference#user-apps>`_

        :param user_id: User Id

        :return: The list of Symphony application entitlements for this user.

        """
        params = {
            'uid': user_id,
            'session_token': await self._auth_session.session_token
        }
        user_app_entitlement_list = await self._app_entitlement_api.v1_admin_user_uid_app_entitlement_list_get(**params)
        return user_app_entitlement_list.value

    @retry
    async def update_user_applications(self, user_id: int, user_app_entitlements: [UserAppEntitlement]):
        """
        Update the application entitlements for a particular user.

        See: `Update User Apps <https://developers.symphony.com/restapi/reference#update-user-apps>`_

        :param user_id:                 User Id
        :param user_app_entitlements:   The list of App Entitlements needs to be updated.

        :return: The updated list of Symphony application entitlements for this user.

        """
        params = {
            'uid': user_id,
            'payload': UserAppEntitlementList(value=user_app_entitlements),
            'session_token': await self._auth_session.session_token
        }
        user_app_entitlement_list \
            = await self._app_entitlement_api.v1_admin_user_uid_app_entitlement_list_post(**params)
        return user_app_entitlement_list.value



"""
    Pod API

    This document refers to Symphony API calls that do not need encryption or decryption of content.  - sessionToken can be obtained by calling the authenticationAPI on the symphony back end and the key manager respectively. Refer to the methods described in authenticatorAPI.yaml. - Actions are defined to be atomic, ie will succeed in their entirety or fail and have changed nothing. - If it returns a 40X status then it will have made no change to the system even if ome subset of the request would have succeeded. - If this contract cannot be met for any reason then this is an error and the response code will be 50X.   # noqa: E501

    The version of the OpenAPI document: 20.12.1
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from symphony.bdk.gen.api_client import ApiClient, Endpoint as _Endpoint
from symphony.bdk.gen.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from symphony.bdk.gen.pod_model.error import Error
from symphony.bdk.gen.pod_model.room_detail import RoomDetail
from symphony.bdk.gen.pod_model.stream import Stream
from symphony.bdk.gen.pod_model.stream_attachment_response import StreamAttachmentResponse
from symphony.bdk.gen.pod_model.stream_filter import StreamFilter
from symphony.bdk.gen.pod_model.stream_list import StreamList
from symphony.bdk.gen.pod_model.user_id_list import UserIdList
from symphony.bdk.gen.pod_model.v2_admin_stream_filter import V2AdminStreamFilter
from symphony.bdk.gen.pod_model.v2_admin_stream_list import V2AdminStreamList
from symphony.bdk.gen.pod_model.v2_membership_list import V2MembershipList
from symphony.bdk.gen.pod_model.v2_room_search_criteria import V2RoomSearchCriteria
from symphony.bdk.gen.pod_model.v2_stream_attributes import V2StreamAttributes
from symphony.bdk.gen.pod_model.v3_room_attributes import V3RoomAttributes
from symphony.bdk.gen.pod_model.v3_room_detail import V3RoomDetail
from symphony.bdk.gen.pod_model.v3_room_search_results import V3RoomSearchResults


class StreamsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __v1_admin_im_create_post(
            self,
            session_token,
            uid_list,
            **kwargs
        ):
            """Create a new single or multi party instant message conversation  # noqa: E501

            At least two user IDs must be provided or an error response will be sent.  The caller is not included in the members of the created chat.  Duplicate users will be included in the membership of the chat but the duplication will be silently ignored.  If there is an existing IM conversation with the same set of participants then the id of that existing stream will be returned.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_admin_im_create_post(session_token, uid_list, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                uid_list (UserIdList): List of (integer) User IDs of participants

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Stream
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['uid_list'] = \
                uid_list
            return self.call_with_http_info(**kwargs)

        self.v1_admin_im_create_post = _Endpoint(
            settings={
                'response_type': (Stream,),
                'auth': [],
                'endpoint_path': '/v1/admin/im/create',
                'operation_id': 'v1_admin_im_create_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'uid_list',
                ],
                'required': [
                    'session_token',
                    'uid_list',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'uid_list':
                        (UserIdList,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                    'uid_list': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_admin_im_create_post
        )

        def __v1_admin_room_id_set_active_post(
            self,
            id,
            active,
            session_token,
            **kwargs
        ):
            """Deactivate or reactivate a chatroom via AC Portal.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_admin_room_id_set_active_post(id, active, session_token, async_req=True)
            >>> result = thread.get()

            Args:
                id (str): Room streamId
                active (bool):
                session_token (str): Session authentication token.

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                RoomDetail
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['id'] = \
                id
            kwargs['active'] = \
                active
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v1_admin_room_id_set_active_post = _Endpoint(
            settings={
                'response_type': (RoomDetail,),
                'auth': [],
                'endpoint_path': '/v1/admin/room/{id}/setActive',
                'operation_id': 'v1_admin_room_id_set_active_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'active',
                    'session_token',
                ],
                'required': [
                    'id',
                    'active',
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'active':
                        (bool,),
                    'session_token':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                    'active': 'active',
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'id': 'path',
                    'active': 'query',
                    'session_token': 'header',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_admin_room_id_set_active_post
        )

        def __v1_admin_stream_id_membership_list_get(
            self,
            id,
            session_token,
            **kwargs
        ):
            """List the current members of an existing stream.  The stream can be of type IM, MIM, or ROOM  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_admin_stream_id_membership_list_get(id, session_token, async_req=True)
            >>> result = thread.get()

            Args:
                id (str): stream Id
                session_token (str): Session authentication token.

            Keyword Args:
                skip (int): Number of items to skip. Default is 0.. [optional]
                limit (int): Maximum number of items to return. Default is 100 and not to exceed 1000.. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V2MembershipList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['id'] = \
                id
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v1_admin_stream_id_membership_list_get = _Endpoint(
            settings={
                'response_type': (V2MembershipList,),
                'auth': [],
                'endpoint_path': '/v1/admin/stream/{id}/membership/list',
                'operation_id': 'v1_admin_stream_id_membership_list_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'session_token',
                    'skip',
                    'limit',
                ],
                'required': [
                    'id',
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'session_token':
                        (str,),
                    'skip':
                        (int,),
                    'limit':
                        (int,),
                },
                'attribute_map': {
                    'id': 'id',
                    'session_token': 'sessionToken',
                    'skip': 'skip',
                    'limit': 'limit',
                },
                'location_map': {
                    'id': 'path',
                    'session_token': 'header',
                    'skip': 'query',
                    'limit': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_admin_stream_id_membership_list_get
        )

        def __v1_im_create_post(
            self,
            session_token,
            uid_list,
            **kwargs
        ):
            """Create a new single or multi party instant message conversation between the caller and specified users.  # noqa: E501

            At least one user ID must be provided or an error response will be sent.  The caller is implicitly included in the members of the created chat.  Duplicate users will be included in the membership of the chat but the duplication will be silently ignored.  If there is an existing IM conversation with the same set of participants then the id of that existing stream will be returned.  This method was incorrectly specified to take a query parameter in version 1.0 of this specification but now expects a JSON array of user IDs in the body of the request.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_im_create_post(session_token, uid_list, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                uid_list (UserIdList): List of (integer) User IDs of participants

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                Stream
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['uid_list'] = \
                uid_list
            return self.call_with_http_info(**kwargs)

        self.v1_im_create_post = _Endpoint(
            settings={
                'response_type': (Stream,),
                'auth': [],
                'endpoint_path': '/v1/im/create',
                'operation_id': 'v1_im_create_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'uid_list',
                ],
                'required': [
                    'session_token',
                    'uid_list',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'uid_list':
                        (UserIdList,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                    'uid_list': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_im_create_post
        )

        def __v1_room_id_set_active_post(
            self,
            id,
            active,
            session_token,
            **kwargs
        ):
            """Deactivate or reactivate a chatroom. At creation, a new chatroom is active.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_room_id_set_active_post(id, active, session_token, async_req=True)
            >>> result = thread.get()

            Args:
                id (str): Room streamId
                active (bool):
                session_token (str): Session authentication token.

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                RoomDetail
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['id'] = \
                id
            kwargs['active'] = \
                active
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v1_room_id_set_active_post = _Endpoint(
            settings={
                'response_type': (RoomDetail,),
                'auth': [],
                'endpoint_path': '/v1/room/{id}/setActive',
                'operation_id': 'v1_room_id_set_active_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'active',
                    'session_token',
                ],
                'required': [
                    'id',
                    'active',
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'active':
                        (bool,),
                    'session_token':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                    'active': 'active',
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'id': 'path',
                    'active': 'query',
                    'session_token': 'header',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_room_id_set_active_post
        )

        def __v1_streams_list_post(
            self,
            session_token,
            **kwargs
        ):
            """Retrieve a list of all streams of which the requesting user is a member, sorted by creation date (ascending).   # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_streams_list_post(session_token, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.

            Keyword Args:
                skip (int): No. of results to skip. . [optional]
                limit (int): Max no. of results to return. If no value is provided, 50 is the default. . [optional]
                filter (StreamFilter): Stream filtering criteria.. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                StreamList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v1_streams_list_post = _Endpoint(
            settings={
                'response_type': (StreamList,),
                'auth': [],
                'endpoint_path': '/v1/streams/list',
                'operation_id': 'v1_streams_list_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'skip',
                    'limit',
                    'filter',
                ],
                'required': [
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'skip':
                        (int,),
                    'limit':
                        (int,),
                    'filter':
                        (StreamFilter,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'skip': 'skip',
                    'limit': 'limit',
                },
                'location_map': {
                    'session_token': 'header',
                    'skip': 'query',
                    'limit': 'query',
                    'filter': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_streams_list_post
        )

        def __v1_streams_sid_attachments_get(
            self,
            sid,
            session_token,
            **kwargs
        ):
            """Get attachments in a particular stream.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_streams_sid_attachments_get(sid, session_token, async_req=True)
            >>> result = thread.get()

            Args:
                sid (str): Stream Id
                session_token (str): Session authentication token.

            Keyword Args:
                since (int): Timestamp of first required attachment. This is a long integer value representing milliseconds since Jan 1 1970 . [optional]
                to (int): Timestamp of last required attachment. This is a long integer value representing milliseconds since Jan 1 1970 . [optional]
                limit (int): Maximum number of attachments to return. Default is 50. Must be a positive integer and must not exceed 100.. [optional]
                sort_dir (str): Attachment date sort direction : ASC or DESC (default to ASC) . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                StreamAttachmentResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['sid'] = \
                sid
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v1_streams_sid_attachments_get = _Endpoint(
            settings={
                'response_type': (StreamAttachmentResponse,),
                'auth': [],
                'endpoint_path': '/v1/streams/{sid}/attachments',
                'operation_id': 'v1_streams_sid_attachments_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'sid',
                    'session_token',
                    'since',
                    'to',
                    'limit',
                    'sort_dir',
                ],
                'required': [
                    'sid',
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'sid':
                        (str,),
                    'session_token':
                        (str,),
                    'since':
                        (int,),
                    'to':
                        (int,),
                    'limit':
                        (int,),
                    'sort_dir':
                        (str,),
                },
                'attribute_map': {
                    'sid': 'sid',
                    'session_token': 'sessionToken',
                    'since': 'since',
                    'to': 'to',
                    'limit': 'limit',
                    'sort_dir': 'sortDir',
                },
                'location_map': {
                    'sid': 'path',
                    'session_token': 'header',
                    'since': 'query',
                    'to': 'query',
                    'limit': 'query',
                    'sort_dir': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_streams_sid_attachments_get
        )

        def __v2_admin_streams_list_post(
            self,
            session_token,
            **kwargs
        ):
            """Retrieve all the streams across the enterprise where the membership of the stream has been modified between a given time range   # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v2_admin_streams_list_post(session_token, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.

            Keyword Args:
                skip (int): Number of items to skip. Default is 0. . [optional]
                limit (int): Maximum number of items to return. Default is 50.  This value cannot exceed 100. . [optional]
                filter (V2AdminStreamFilter): Stream filtering criteria.. [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V2AdminStreamList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v2_admin_streams_list_post = _Endpoint(
            settings={
                'response_type': (V2AdminStreamList,),
                'auth': [],
                'endpoint_path': '/v2/admin/streams/list',
                'operation_id': 'v2_admin_streams_list_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'skip',
                    'limit',
                    'filter',
                ],
                'required': [
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'skip':
                        (int,),
                    'limit':
                        (int,),
                    'filter':
                        (V2AdminStreamFilter,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'skip': 'skip',
                    'limit': 'limit',
                },
                'location_map': {
                    'session_token': 'header',
                    'skip': 'query',
                    'limit': 'query',
                    'filter': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v2_admin_streams_list_post
        )

        def __v2_streams_sid_info_get(
            self,
            sid,
            session_token,
            **kwargs
        ):
            """Get information about a partcular stream.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v2_streams_sid_info_get(sid, session_token, async_req=True)
            >>> result = thread.get()

            Args:
                sid (str): Stream Id
                session_token (str): Session authentication token.

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V2StreamAttributes
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['sid'] = \
                sid
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v2_streams_sid_info_get = _Endpoint(
            settings={
                'response_type': (V2StreamAttributes,),
                'auth': [],
                'endpoint_path': '/v2/streams/{sid}/info',
                'operation_id': 'v2_streams_sid_info_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'sid',
                    'session_token',
                ],
                'required': [
                    'sid',
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'sid':
                        (str,),
                    'session_token':
                        (str,),
                },
                'attribute_map': {
                    'sid': 'sid',
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'sid': 'path',
                    'session_token': 'header',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v2_streams_sid_info_get
        )

        def __v3_room_create_post(
            self,
            session_token,
            payload,
            **kwargs
        ):
            """Create a new chatroom.  # noqa: E501

            Create a new chatroom.  If no  attributes are specified, the room is created as a private chatroom.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v3_room_create_post(session_token, payload, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                payload (V3RoomAttributes):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V3RoomDetail
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['payload'] = \
                payload
            return self.call_with_http_info(**kwargs)

        self.v3_room_create_post = _Endpoint(
            settings={
                'response_type': (V3RoomDetail,),
                'auth': [],
                'endpoint_path': '/v3/room/create',
                'operation_id': 'v3_room_create_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'payload',
                ],
                'required': [
                    'session_token',
                    'payload',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'payload':
                        (V3RoomAttributes,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                    'payload': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v3_room_create_post
        )

        def __v3_room_id_info_get(
            self,
            id,
            session_token,
            **kwargs
        ):
            """Get information about a partcular chatroom.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v3_room_id_info_get(id, session_token, async_req=True)
            >>> result = thread.get()

            Args:
                id (str): Room streamId
                session_token (str): Session authentication token.

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V3RoomDetail
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['id'] = \
                id
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v3_room_id_info_get = _Endpoint(
            settings={
                'response_type': (V3RoomDetail,),
                'auth': [],
                'endpoint_path': '/v3/room/{id}/info',
                'operation_id': 'v3_room_id_info_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'session_token',
                ],
                'required': [
                    'id',
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'session_token':
                        (str,),
                },
                'attribute_map': {
                    'id': 'id',
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'id': 'path',
                    'session_token': 'header',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v3_room_id_info_get
        )

        def __v3_room_id_update_post(
            self,
            id,
            session_token,
            payload,
            **kwargs
        ):
            """Update the attributes of an existing chatroom.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v3_room_id_update_post(id, session_token, payload, async_req=True)
            >>> result = thread.get()

            Args:
                id (str): Room streamId
                session_token (str): Session authentication token.
                payload (V3RoomAttributes):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V3RoomDetail
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['id'] = \
                id
            kwargs['session_token'] = \
                session_token
            kwargs['payload'] = \
                payload
            return self.call_with_http_info(**kwargs)

        self.v3_room_id_update_post = _Endpoint(
            settings={
                'response_type': (V3RoomDetail,),
                'auth': [],
                'endpoint_path': '/v3/room/{id}/update',
                'operation_id': 'v3_room_id_update_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'id',
                    'session_token',
                    'payload',
                ],
                'required': [
                    'id',
                    'session_token',
                    'payload',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'id':
                        (str,),
                    'session_token':
                        (str,),
                    'payload':
                        (V3RoomAttributes,),
                },
                'attribute_map': {
                    'id': 'id',
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'id': 'path',
                    'session_token': 'header',
                    'payload': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v3_room_id_update_post
        )

        def __v3_room_search_post(
            self,
            session_token,
            query,
            **kwargs
        ):
            """Search rooms according to the specified criteria.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v3_room_search_post(session_token, query, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                query (V2RoomSearchCriteria): The search query object.

            Keyword Args:
                skip (int): No. of results to skip. . [optional]
                limit (int): Max no. of results to return. If no value is provided, 50 is the default. Must be a positive integer and must not exceed 100 . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                V3RoomSearchResults
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['query'] = \
                query
            return self.call_with_http_info(**kwargs)

        self.v3_room_search_post = _Endpoint(
            settings={
                'response_type': (V3RoomSearchResults,),
                'auth': [],
                'endpoint_path': '/v3/room/search',
                'operation_id': 'v3_room_search_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'query',
                    'skip',
                    'limit',
                ],
                'required': [
                    'session_token',
                    'query',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'query':
                        (V2RoomSearchCriteria,),
                    'skip':
                        (int,),
                    'limit':
                        (int,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'skip': 'skip',
                    'limit': 'limit',
                },
                'location_map': {
                    'session_token': 'header',
                    'query': 'body',
                    'skip': 'query',
                    'limit': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v3_room_search_post
        )



"""This module takes care of creating generators from paginated endpoints, so that user do not have to care about
making several calls to the same endpoint with the correct pagination values.
"""
from typing import AsyncGenerator, TypeVar, Callable, Awaitable, Tuple

T = TypeVar('T')


async def offset_based_pagination(func: Callable[[int, int], Awaitable[T]],
                                  chunk_size=50, max_number=None) -> AsyncGenerator[T, None]:
    """Creates an asynchronous generator from a paginated endpoint. The generator makes the call to the underlying
    endpoint `func` until the `max_number` of items is reached or results are exhausted (i.e. `func` is None or empty).

    :param func: a coroutine taking two int parameters: `skip` (a.k.a offset) and `limit` (max number for items to
      retrieve in one call) and returning a list.
    :param chunk_size: the maximum number of elements to retrieve in one call.
    :param max_number: the maximum total number of items to retrieve. If not specified or set to None, it will fetch all
      items until we retrieved all elements.
    :return: an asynchronous generator of elements which makes the calls to `func` with the correct parameters.
    """
    if max_number is not None and max_number <= 0:
        return

    skip = 0
    item_count = 0

    chunk = await func(skip, chunk_size)
    while chunk:
        for item in chunk:
            yield item

            item_count += 1
            if max_number is not None and item_count == max_number:
                # max_number items already retrieved
                return

        if len(chunk) < chunk_size:
            # received chunk has less elements than the sent chunk size: we are already at the end
            return

        skip += chunk_size
        chunk = await func(skip, chunk_size)


async def cursor_based_pagination(func: Callable[[int, str], Awaitable[Tuple[T, str]]],
                                  chunk_size=100, max_number=None) -> AsyncGenerator[T, None]:
    """Creates an asynchronous generator from a cursor based endpoint. The generator makes the call to the underlying
    endpoint `func` until the `max_number` of items is reached or results are exhausted (i.e. cursor returned by `func`
    is None).

    :param func: a coroutine taking two int parameters: limit (max number for items to retrieve
      in one call) and after (for cursor based pagination).
      It must return a tuple `(result, after)` where `result` is a list containing the actual results in the chunk and
      `after` is the new cursor which will be passed to the next call to `func`.
    :param chunk_size: the maximum number of elements to retrieve in one call.
    :param max_number: the maximum total number of items to retrieve. If not specified or set to None, it will fetch all
      items until we retrieved all elements.
    :return: an asynchronous generator of elements which makes the calls to `func` with the correct parameters.
    """
    if max_number is not None and max_number <= 0:
        return

    after = None
    item_count = 0

    while True:
        (result, after) = await func(chunk_size, after)
        if result is None:
            result = []

        for item in result:
            yield item

            item_count += 1
            if max_number is not None and item_count == max_number:
                # max_number items already retrieved
                return

        if after is None:
            # we exhausted the results
            break



"""Module containing OBO authenticator classes.
"""
from abc import ABC, abstractmethod

from symphony.bdk.core.auth.auth_session import OboAuthSession
from symphony.bdk.core.auth.jwt_helper import create_signed_jwt
from symphony.bdk.core.config.model.bdk_app_config import BdkAppConfig
from symphony.bdk.core.config.model.bdk_retry_config import BdkRetryConfig
from symphony.bdk.core.retry import retry
from symphony.bdk.core.retry.strategy import authentication_retry
from symphony.bdk.gen.auth_api.certificate_authentication_api import CertificateAuthenticationApi
from symphony.bdk.gen.login_api.authentication_api import AuthenticationApi
from symphony.bdk.gen.login_model.authenticate_request import AuthenticateRequest


class OboAuthenticator(ABC):
    """Obo authentication service.
    """

    unauthorized_message = \
        "Extension Application is not authorized to authenticate in OBO mode. Check if credentials are valid."

    @abstractmethod
    async def retrieve_obo_session_token_by_user_id(self, user_id: int) -> str:
        """Retrieve the OBO session token by username.

        :param user_id: User Id.
        :return: The obo session token.
        :raise AuthUnauthorizedError: if session token cannot be retrieved
        """

    @abstractmethod
    async def retrieve_obo_session_token_by_username(self, username: str) -> str:
        """Retrieve the OBO session token by username.

        :param username: Username
        :return: The obo session token.
        :raise AuthUnauthorizedError: if session token cannot be retrieved
        """

    def authenticate_by_username(self, username: str) -> OboAuthSession:
        """Authenticate On-Behalf-Of user by username.

        :param username: Username
        :return: the OBO authentication session.
        """
        return OboAuthSession(self, username=username)

    def authenticate_by_user_id(self, user_id: int) -> OboAuthSession:
        """Authenticate On-Behalf-Of user by user id.

        :param user_id: User Id
        :return: the OBO authentication session.
        """
        return OboAuthSession(self, user_id=user_id)


class OboAuthenticatorRsa(OboAuthenticator):
    """Obo authenticator RSA implementation.
    """

    def __init__(self, app_config: BdkAppConfig, authentication_api: AuthenticationApi, retry_config: BdkRetryConfig):
        self._app_config = app_config
        self._authentication_api = authentication_api
        self._retry_config = retry_config

    async def retrieve_obo_session_token_by_user_id(self, user_id: int) -> str:
        """Retrieve the OBO session token by user id.

        :param user_id: User Id.
        :return: The obo session token.
        :raise AuthUnauthorizedError: if session token cannot be retrieved
        """
        app_session_token = await self._authenticate_and_retrieve_app_session_token()
        return await self._authenticate_by_user_id(app_session_token, user_id=user_id)

    async def retrieve_obo_session_token_by_username(self, username: str) -> str:
        """Retrieve the OBO session token by username.

        :param username: Username
        :return: The obo session token.
        :raise AuthUnauthorizedError: if session token cannot be retrieved
        """
        app_session_token = await self._authenticate_and_retrieve_app_session_token()
        return await self._authenticate_by_username(app_session_token, username=username)

    @retry(retry=authentication_retry)
    async def _authenticate_and_retrieve_app_session_token(self) -> str:
        jwt = create_signed_jwt(self._app_config.private_key, self._app_config.app_id)
        req = AuthenticateRequest(token=jwt)

        token = await self._authentication_api.pubkey_app_authenticate_post(req)
        return token.token

    @retry(retry=authentication_retry)
    async def _authenticate_by_user_id(self, app_session_token, user_id) -> str:
        token = await self._authentication_api.pubkey_app_user_user_id_authenticate_post(
            session_token=app_session_token, user_id=user_id)
        return token.token

    @retry(retry=authentication_retry)
    async def _authenticate_by_username(self, app_session_token, username) -> str:
        token = await self._authentication_api.pubkey_app_username_username_authenticate_post(
            session_token=app_session_token, username=username)
        return token.token


class OboAuthenticatorCert(OboAuthenticator):
    """Obo authenticator Certificate implementation.
    """

    def __init__(self, certificate_authenticator_api: CertificateAuthenticationApi, retry_config: BdkRetryConfig):
        self._authentication_api = certificate_authenticator_api
        self._retry_config = retry_config

    @retry(retry=authentication_retry)
    async def retrieve_obo_session_token_by_user_id(self, user_id: int) -> str:
        """Retrieve the OBO session token by username.

        :param user_id: User Id.
        :return: The obo session token.
        :raise AuthUnauthorizedError: if session token cannot be retrieved
        """
        app_session_token = await self._retrieve_app_session_token()
        obo_auth = await self._authentication_api.v1_app_user_uid_authenticate_post(session_token=app_session_token,
                                                                                    uid=user_id)
        return obo_auth.session_token

    @retry(retry=authentication_retry)
    async def retrieve_obo_session_token_by_username(self, username: str) -> str:
        """Retrieve the OBO session token by username.

        :param username: Username
        :return: The obo session token.
        :raise AuthUnauthorizedError: if session token cannot be retrieved
        """
        app_session_token = await self._retrieve_app_session_token()
        obo_auth = await self._authentication_api.v1_app_username_username_authenticate_post(
            session_token=app_session_token, username=username)
        return obo_auth.session_token

    @retry(retry=authentication_retry)
    async def _retrieve_app_session_token(self) -> str:
        token = await self._authentication_api.v1_app_authenticate_post()
        return token.token



class BdkRsaKeyConfig:
    """Class containing the bot's RSA Key configuration
    """
    def __init__(self, path=None, content=""):
        self._path = path
        self._content = content

    @property
    def path(self):
        """Path to rsa private key file."""
        return self._path

    @property
    def content(self):
        """rsa private key content string"""
        return self._content

    @content.setter
    def content(self, rsa_key_content):
        """Sets rsa content and overrides path to None to keep key's config valid

        :param rsa_key_content: rsa private key content
        """
        self._content = rsa_key_content
        self._path = None

    @path.setter
    def path(self, rsa_key_path):
        """Sets rsa path and overrides content to None to keep key's config valid
        :param rsa_key_path: rsa private key path
        """
        self._path = rsa_key_path
        self._content = None

    def is_configured(self) -> bool:
        """"Check if the RSA authentication is configured or not

        :return: true if the RSA authentication is configured
        """
        return self._path is not None or self._content != ""

    def is_valid(self) -> bool:
        """Check if the RSA configuration is valid.
        If both of private key path and content, the configuration is invalid.

        :return: true if the RSA configuration is valid.
        """
        return not (self._path is not None and self._content != "")

    def get_private_key_content(self) -> str:
        """Loads the private key content.
        If the path is set, it loads the file content, otherwise it returns the content.

        :return: private key content as string.
        """
        return self._load_key_from_path() \
            if self._path is not None else self._content

    def _load_key_from_path(self):
        with open(self._path, "r") as file:
            private_key_content = file.readlines()
            key = "".join(private_key_content)
            return key



from symphony.bdk.core.config.model.bdk_app_config import BdkAppConfig
from symphony.bdk.core.config.model.bdk_bot_config import BdkBotConfig
from symphony.bdk.core.config.model.bdk_client_config import BdkClientConfig
from symphony.bdk.core.config.model.bdk_retry_config import BdkRetryConfig
from symphony.bdk.core.config.model.bdk_server_config import BdkServerConfig
from symphony.bdk.core.config.model.bdk_ssl_config import BdkSslConfig
from symphony.bdk.core.config.model.bdk_datafeed_config import BdkDatafeedConfig


class BdkConfig(BdkServerConfig):
    """Class containing the whole BDK configuration.
    """

    def __init__(self, **config):
        """

        :param config: the dict containing the server configuration parameters.
        """
        super().__init__(scheme=config.get("scheme"), host=config.get("host"), port=config.get("port"),
                         context=config.get("context"), proxy=config.get("proxy"),
                         default_headers=config.get("defaultHeaders"))
        self.agent = BdkClientConfig(self, config.get("agent"))
        self.pod = BdkClientConfig(self, config.get("pod"))
        self.key_manager = BdkClientConfig(self, config.get("keyManager"))
        self.session_auth = BdkClientConfig(self, config.get("sessionAuth"))
        self.bot = BdkBotConfig(config.get("bot"))
        self.ssl = BdkSslConfig(config.get("ssl"))
        self.app = BdkAppConfig(config.get("app"))
        self.datafeed = BdkDatafeedConfig(config.get("datafeed"))
        self.retry = BdkRetryConfig(config.get("retry"))

    def is_bot_configured(self) -> bool:
        """

        :return: true if bot service account is specified in the configuration.
        """
        return self.bot.username is not None

    def is_app_configured(self) -> bool:
        """

        :return: true if app id is specified in the configuration.
        """
        return self.app.app_id is not None



"""
    Pod API

    This document refers to Symphony API calls that do not need encryption or decryption of content.  - sessionToken can be obtained by calling the authenticationAPI on the symphony back end and the key manager respectively. Refer to the methods described in authenticatorAPI.yaml. - Actions are defined to be atomic, ie will succeed in their entirety or fail and have changed nothing. - If it returns a 40X status then it will have made no change to the system even if ome subset of the request would have succeeded. - If this contract cannot be met for any reason then this is an error and the response code will be 50X.   # noqa: E501

    The version of the OpenAPI document: 20.12.1
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from symphony.bdk.gen.api_client import ApiClient, Endpoint as _Endpoint
from symphony.bdk.gen.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from symphony.bdk.gen.pod_model.error import Error
from symphony.bdk.gen.pod_model.success_response import SuccessResponse
from symphony.bdk.gen.pod_model.user_connection import UserConnection
from symphony.bdk.gen.pod_model.user_connection_list import UserConnectionList
from symphony.bdk.gen.pod_model.user_connection_request import UserConnectionRequest


class ConnectionApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __v1_connection_accept_post(
            self,
            session_token,
            connection_request,
            **kwargs
        ):
            """Accept the connection request for the requesting user  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_connection_accept_post(session_token, connection_request, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                connection_request (UserConnectionRequest):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                UserConnection
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['connection_request'] = \
                connection_request
            return self.call_with_http_info(**kwargs)

        self.v1_connection_accept_post = _Endpoint(
            settings={
                'response_type': (UserConnection,),
                'auth': [],
                'endpoint_path': '/v1/connection/accept',
                'operation_id': 'v1_connection_accept_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'connection_request',
                ],
                'required': [
                    'session_token',
                    'connection_request',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'connection_request':
                        (UserConnectionRequest,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                    'connection_request': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_connection_accept_post
        )

        def __v1_connection_create_post(
            self,
            session_token,
            connection_request,
            **kwargs
        ):
            """Sends an invitation to connect with another user  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_connection_create_post(session_token, connection_request, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                connection_request (UserConnectionRequest):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                UserConnection
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['connection_request'] = \
                connection_request
            return self.call_with_http_info(**kwargs)

        self.v1_connection_create_post = _Endpoint(
            settings={
                'response_type': (UserConnection,),
                'auth': [],
                'endpoint_path': '/v1/connection/create',
                'operation_id': 'v1_connection_create_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'connection_request',
                ],
                'required': [
                    'session_token',
                    'connection_request',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'connection_request':
                        (UserConnectionRequest,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                    'connection_request': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_connection_create_post
        )

        def __v1_connection_list_get(
            self,
            session_token,
            **kwargs
        ):
            """List of requesting user's connection  # noqa: E501

            This retrieves all connections of the requesting user. (i.e. both connections in which the requesting user is the sender and those in which the requesting user is the inivtee) By default, if you haven't specified the connection status to filter on, this call will only return results for both \"pending_incoming\" and \"pending_outgoing\". You can optionally filter by userIds to further restrict the results of a specific connection status. If the users are in the same private pod, the users have an implicit connection status of \"accepted\". Those users will not be returned in the response if you don't specify the connection status as \"accepted\" (default is \"pending\") and the explicit userIds in the request.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_connection_list_get(session_token, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.

            Keyword Args:
                status (str): Filter the connection list based on the connection status. The connection status can only be pending_incoming, pending_outgoing, accepted, rejected, or all (all of the above) . [optional]
                user_ids (str): The userIds parameter should be specified as a comma delimited list of user ids and can be used to restrict the results of a specific connection. Note that this is particularly important if the caller intends to retrieve results for implicit connection (user within the same pod). Implicit connections will not be included in the response if userId is not provided. . [optional]
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                UserConnectionList
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            return self.call_with_http_info(**kwargs)

        self.v1_connection_list_get = _Endpoint(
            settings={
                'response_type': (UserConnectionList,),
                'auth': [],
                'endpoint_path': '/v1/connection/list',
                'operation_id': 'v1_connection_list_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'status',
                    'user_ids',
                ],
                'required': [
                    'session_token',
                ],
                'nullable': [
                ],
                'enum': [
                    'status',
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                    ('status',): {

                        "PENDING_INCOMING": "PENDING_INCOMING",
                        "PENDING_OUTGOING": "PENDING_OUTGOING",
                        "ACCEPTED": "ACCEPTED",
                        "REJECTED": "REJECTED",
                        "ALL": "ALL"
                    },
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'status':
                        (str,),
                    'user_ids':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'status': 'status',
                    'user_ids': 'userIds',
                },
                'location_map': {
                    'session_token': 'header',
                    'status': 'query',
                    'user_ids': 'query',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_connection_list_get
        )

        def __v1_connection_reject_post(
            self,
            session_token,
            connection_request,
            **kwargs
        ):
            """Reject the connection request for the requesting user.  # noqa: E501

            Reject the connection between the requesting user and request sender. If both users are in the same private pod, an error will be returned because both users have an implicit connection which cannot be rejected.   # noqa: E501
            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_connection_reject_post(session_token, connection_request, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                connection_request (UserConnectionRequest):

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                UserConnection
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['connection_request'] = \
                connection_request
            return self.call_with_http_info(**kwargs)

        self.v1_connection_reject_post = _Endpoint(
            settings={
                'response_type': (UserConnection,),
                'auth': [],
                'endpoint_path': '/v1/connection/reject',
                'operation_id': 'v1_connection_reject_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'connection_request',
                ],
                'required': [
                    'session_token',
                    'connection_request',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'connection_request':
                        (UserConnectionRequest,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                },
                'location_map': {
                    'session_token': 'header',
                    'connection_request': 'body',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [
                    'application/json'
                ]
            },
            api_client=api_client,
            callable=__v1_connection_reject_post
        )

        def __v1_connection_user_uid_remove_post(
            self,
            session_token,
            uid,
            **kwargs
        ):
            """Removes a connection with a user.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_connection_user_uid_remove_post(session_token, uid, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                uid (int): User ID as a decimal integer 

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                SuccessResponse
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['uid'] = \
                uid
            return self.call_with_http_info(**kwargs)

        self.v1_connection_user_uid_remove_post = _Endpoint(
            settings={
                'response_type': (SuccessResponse,),
                'auth': [],
                'endpoint_path': '/v1/connection/user/{uid}/remove',
                'operation_id': 'v1_connection_user_uid_remove_post',
                'http_method': 'POST',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'uid',
                ],
                'required': [
                    'session_token',
                    'uid',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'uid':
                        (int,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'uid': 'uid',
                },
                'location_map': {
                    'session_token': 'header',
                    'uid': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_connection_user_uid_remove_post
        )

        def __v1_connection_user_user_id_info_get(
            self,
            session_token,
            user_id,
            **kwargs
        ):
            """The status of the connection invitation to another user.  # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = pod_api.v1_connection_user_user_id_info_get(session_token, user_id, async_req=True)
            >>> result = thread.get()

            Args:
                session_token (str): Session authentication token.
                user_id (str): user Id

            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (float/tuple): timeout setting for this request. If one
                    number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                UserConnection
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['session_token'] = \
                session_token
            kwargs['user_id'] = \
                user_id
            return self.call_with_http_info(**kwargs)

        self.v1_connection_user_user_id_info_get = _Endpoint(
            settings={
                'response_type': (UserConnection,),
                'auth': [],
                'endpoint_path': '/v1/connection/user/{userId}/info',
                'operation_id': 'v1_connection_user_user_id_info_get',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    'session_token',
                    'user_id',
                ],
                'required': [
                    'session_token',
                    'user_id',
                ],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                    'session_token':
                        (str,),
                    'user_id':
                        (str,),
                },
                'attribute_map': {
                    'session_token': 'sessionToken',
                    'user_id': 'userId',
                },
                'location_map': {
                    'session_token': 'header',
                    'user_id': 'path',
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    'application/json'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__v1_connection_user_user_id_info_get
        )



import json
from typing import List, Tuple, Union, IO

from symphony.bdk.core.service.exception import MessageCreationError

MESSAGE_ML_END_TAG = "</messageML>"
MESSAGE_ML_START_TAG = "<messageML>"


class Message:
    """Class holding message information, used in
    :py:class:`~symphony.bdk.core.service.message.message_service.MessageService` to send a message.
    To know more about the format, see
    `Create Message <https://developers.symphony.com/restapi/reference#create-message-v4>`_.
    """

    def __init__(self, content: str, data=None,
                 attachments: List[Union[IO, Tuple[IO, IO]]] = None, version: str = ""):
        """Builds a message.

        :param content: the MessageML content to be sent. This is mandatory
          If there is no <messageML> tags, they will be added to the content.
        :param data: an object (e.g. dict) that will be serialized into JSON using ``json.dumps``
        :param attachments: list of attachments or list of (attachment, previews).
          These must be opened files either in binary or text mode.
          Previews are optional but if present, all attachments must have a preview.
        :param version: Optional message version in the format "major.minor".
          If empty, defaults to the latest supported version.
        :raise MessageCreationError: if content is None,
          or if number of previews different than the number of attachments (in case there is at least one preview).
        """
        self._content = self._get_content(content)
        self._data = "" if data is None else json.dumps(data)
        self._version = version
        self._attachments, self._previews = self._get_attachments_and_previews(attachments)

    @property
    def content(self) -> str:
        """Message content

        :return: the messageML content
        """
        return self._content

    @property
    def data(self) -> str:
        """Message data

        :return: the data as a JSON string
        """
        return self._data

    @property
    def version(self) -> str:
        """Message format version.

        :return: the message format version
        """
        return self._version

    @property
    def attachments(self) -> List[IO]:
        """List of attachments

        :return: the list of attachments
        """
        return self._attachments

    @property
    def previews(self) -> List[IO]:
        """List of previews

        :return: the list of previews
        """
        return self._previews

    @staticmethod
    def _get_content(content):
        if content is None:
            raise MessageCreationError("Message content is mandatory")
        if not content.startswith(MESSAGE_ML_START_TAG) and not content.endswith(MESSAGE_ML_END_TAG):
            return MESSAGE_ML_START_TAG + content + MESSAGE_ML_END_TAG
        return content

    @staticmethod
    def _get_attachments_and_previews(attachments_previews):
        if attachments_previews is None:
            return [], []

        attachments = []
        previews = []
        for item in attachments_previews:
            if isinstance(item, tuple) and len(item) >= 1:
                attachments.append(item[0])
                if len(item) >= 2:
                    previews.append(item[1])
            else:
                attachments.append(item)

        if previews and len(previews) != len(attachments):
            raise MessageCreationError("Message should contain either no preview or as many previews as attachments")

        return attachments, previews



"""Module responsible for managing the trace ID sent as X-Trace-Id header and logged under the `trace_id` variable
"""
import logging
import random
import string
from contextvars import ContextVar

TRACE_ID_LENGTH = 6

X_TRACE_ID = "X-Trace-Id"

HEADER_ARG_INDEX = 4


def setup_trace_id_log_record_factory():
    """Adds the `trace_id` variable to the log records.

    :return: the updated log record factory which adds the `trace_id` variable with the current trace id value
    """
    old_factory = logging.getLogRecordFactory()

    def trace_id_log_factory(*args, **kwargs):
        record = old_factory(*args, **kwargs)
        record.trace_id = DistributedTracingContext.get_trace_id()

        return record

    logging.setLogRecordFactory(trace_id_log_factory)


def add_x_trace_id(func):
    """Decorator of ApiClient.__call_api function so that it adds the `X-Trace-Id` header
    with the current trace id value to the HTTP call.

    :param func: the function to be decorated
    :return: the decorated function
    """

    async def add_x_trace_id_header(*args, **kwargs):
        if not DistributedTracingContext.is_trace_id_set_by_user():
            DistributedTracingContext.set_new_trace_id()

        if DistributedTracingContext.has_trace_id() and len(args) > HEADER_ARG_INDEX:
            args[HEADER_ARG_INDEX][X_TRACE_ID] = DistributedTracingContext.get_trace_id()

        return await func(*args, **kwargs)

    return add_x_trace_id_header


class DistributedTracingContext:
    """Class to manage the tracing id context.
    """

    _trace_id_context = ContextVar("_trace_id_context")
    _is_trace_id_set_by_user = False

    @classmethod
    def get_trace_id(cls) -> str:
        """Gets the current trace id.

        :return: the current trace id or empty string if not set
        """
        return cls._trace_id_context.get("")

    @classmethod
    def has_trace_id(cls) -> bool:
        """Checks whether a trace id has been set.

        :return: True if a trace id has been set, False otherwise
        """
        return cls.get_trace_id() != ""

    @classmethod
    def is_trace_id_set_by_user(cls) -> bool:
        """Checks whether the trace id has been manually set using :func:`~set_trace_id`.

        :return: True if a trace id has been manually set, False otherwise
        """
        return cls._is_trace_id_set_by_user

    @classmethod
    def set_trace_id(cls, trace_id: str) -> None:
        """Sets the trace id to a new value.

        :param trace_id: string value, should not be None or empty
        :return: None
        """
        cls._trace_id_context.set(trace_id)
        cls._is_trace_id_set_by_user = True

    @classmethod
    def set_new_trace_id(cls) -> None:
        """Sets the trace id to a newly generated value.

        :return: None
        """
        trace_id = "".join(random.choices(string.ascii_letters + string.digits, k=TRACE_ID_LENGTH))
        cls._trace_id_context.set(trace_id)

    @classmethod
    def clear(cls) -> None:
        """Clears the trace id value.

        :return: None
        """
        cls._trace_id_context.set("")
        cls._is_trace_id_set_by_user = False



from symphony.bdk.core.config.model.bdk_server_config import BdkServerConfig, BdkProxyConfig


class BdkClientConfig(BdkServerConfig):
    """Class containing a client configuration:
    - scheme (e.g. 'https')
    - host
    - port
    - context (path to be appended after the port).
    This is used to build the URL when making HTTP calls.
    URL will be built as follows: {scheme}://{host}:{port}/{context}
    """

    def __init__(self, parent_config, config):
        """

        :param parent_config: the parent configuration of type BdkConfig
        :param config: client configuration parameters of type dict
        """
        if config is None:
            config = {}

        self._scheme = config.get("scheme")
        self._port = config.get("port")
        self._host = config.get("host")
        self._context = config.get("context")
        self._proxy = BdkProxyConfig(**config.get("proxy")) if "proxy" in config else None
        self._default_headers = config.get("defaultHeaders") if "defaultHeaders" in config else None
        self.parent_config = parent_config

    @property
    def scheme(self):
        """Return the applicable scheme: either the one configured at child level (e.g. 'pod') or at global level.

        :return: the applicable scheme (e.g. 'https')
        """
        return self._self_or_parent(self._scheme, self.parent_config.scheme)

    @property
    def port(self):
        """Return the applicable port: either the one configured at child level (e.g. 'pod') or at global level.

        :return: the applicable port (e.g. 443)
        """
        return self._self_or_parent(self._port, self.parent_config.port)

    @property
    def host(self):
        """Return the applicable host: either the one configured at child level (e.g. 'pod') or at global level.

        :return: the applicable host (e.g. 'acme.symphony.com')
        """
        return self._self_or_parent(self._host, self.parent_config.host)

    @property
    def context(self):
        """Return the applicable context path: either the one configured at child level (e.g. 'pod') or at global level.

        :return: the applicable context path
        """
        return self._self_or_parent(self._context, self.parent_config.context)

    @property
    def proxy(self):
        """Return the applicable proxy information: either the one configured at child level (e.g. 'pod')
        or at global level.

        :return: the applicable proxy information
        """
        return self._self_or_parent(self._proxy, self.parent_config.proxy)

    @property
    def default_headers(self):
        """Return the applicable default headers: either the one configured at child level (e.g. 'pod')
        or at global level.

        :return: the applicable default headers
        """
        return self._self_or_parent(self._default_headers, self.parent_config.default_headers)

    @staticmethod
    def _self_or_parent(instance_value, parent_value):
        """Get the parent configuration field if the current client's field is not defined

        :param instance_value: current client field
        :param parent_value: parent configuration field
        :return: the value at child level if set or at parent level otherwise
        """
        return instance_value if instance_value is not None else parent_value



# flake8: noqa

"""
    Login API

    For bots and other on-premise processes to authenticate. Once authenticated, the bot will be able to use the methods described in serviceAPI.yaml and agentAPI.yaml.  Authentication requests will expect the user to pass a token containing user identification information and signed by the user's private key  There will be two implementations of this API, one on your Pod and one on the Key Manager. In order to fully authenticate, an API client will have to call both of these implementations and pass both of the session tokens returned as headers in all subsequent requests to the Symphony API.   # noqa: E501

    The version of the OpenAPI document: 20.10.0
    Generated by: https://openapi-generator.tech
"""


__version__ = "1.0.0"

# import ApiClient
from symphony.bdk.gen.api_client import ApiClient

# import Configuration
from symphony.bdk.gen.configuration import Configuration

# import exceptions
from symphony.bdk.gen.exceptions import OpenApiException
from symphony.bdk.gen.exceptions import ApiAttributeError
from symphony.bdk.gen.exceptions import ApiTypeError
from symphony.bdk.gen.exceptions import ApiValueError
from symphony.bdk.gen.exceptions import ApiKeyError
from symphony.bdk.gen.exceptions import ApiException



"""Module for instantiating various authenticator objects.
"""

from symphony.bdk.core.auth.bot_authenticator import BotAuthenticator, BotAuthenticatorRsa, BotAuthenticatorCert
from symphony.bdk.core.auth.exception import AuthInitializationError
from symphony.bdk.core.auth.ext_app_authenticator import ExtensionAppAuthenticator, ExtensionAppAuthenticatorRsa, \
    ExtensionAppAuthenticatorCert
from symphony.bdk.core.auth.obo_authenticator import OboAuthenticator, OboAuthenticatorRsa, OboAuthenticatorCert
from symphony.bdk.core.client.api_client_factory import ApiClientFactory
from symphony.bdk.core.config.model.bdk_config import BdkConfig
from symphony.bdk.gen.auth_api.certificate_authentication_api import CertificateAuthenticationApi
from symphony.bdk.gen.auth_api.certificate_pod_api import CertificatePodApi
from symphony.bdk.gen.login_api.authentication_api import AuthenticationApi
from symphony.bdk.gen.pod_api.pod_api import PodApi


class AuthenticatorFactory:
    """Authenticator Factory class

    Provides new instances for the main authenticators:
        - :class:`symphony.bdk.core.auth.bot_authenticator.BotAuthenticator`: to authenticate the main Bot service
          account
        - :class:`symphony.bdk.core.auth.obo_authenticator.OboAuthenticator`: to perform on-behalf-of authentication
        - :class:`symphony.bdk.core.auth.ext_app_authenticator.ExtensionAppAuthenticator`: to perform extension
          application authentication
    """

    def __init__(self, config: BdkConfig, api_client_factory: ApiClientFactory):
        """

        :param config: the bot configuration
        :param api_client_factory: the :class:`symphony.bdk.core.client.api_client_factory.ApiClientFactory` instance to
          create the BotAuthenticator from.
        """
        self._config = config
        self._api_client_factory = api_client_factory

    def get_bot_authenticator(self) -> BotAuthenticator:
        """Creates a new instance of a Bot Authenticator service.

        :return: a new :class:`symphony.bdk.core.auth.bot_authenticator.BotAuthenticator` instance.
        """
        if self._config.bot.is_rsa_configuration_valid():
            return BotAuthenticatorRsa(
                self._config.bot,
                self._api_client_factory.get_login_client(),
                self._api_client_factory.get_relay_client(),
                self._config.retry
            )
        if self._config.bot.is_certificate_configuration_valid():
            return BotAuthenticatorCert(
                self._api_client_factory.get_session_auth_client(),
                self._api_client_factory.get_key_auth_client(),
                self._config.retry
            )
        raise AuthInitializationError("RSA or certificate authentication should be configured. "
                                      "Only one field among private key path or content should be configured "
                                      "for bot RSA authentication. "
                                      "The path field should be specified for bot certificate authentication.")

    def get_obo_authenticator(self) -> OboAuthenticator:
        """Creates a new instance of a Obo Authenticator service.

        :return: a new :class:`symphony.bdk.core.auth.obo_authenticator.OboAuthenticator` instance.
        """
        app_config = self._config.app
        if app_config.is_rsa_configuration_valid():
            return OboAuthenticatorRsa(
                app_config,
                AuthenticationApi(self._api_client_factory.get_login_client()),
                self._config.retry
            )
        if app_config.is_certificate_configuration_valid():
            authentication_api = CertificateAuthenticationApi(self._api_client_factory.get_app_session_auth_client())
            return OboAuthenticatorCert(authentication_api, self._config.retry)
        raise AuthInitializationError("Application under 'app' field should be configured with a private key or "
                                      "a certificate in order to use OBO authentication.")

    def get_extension_app_authenticator(self) -> ExtensionAppAuthenticator:
        """Creates a new instance of an extension app authenticator service.

        :return: a new :class:`symphony.bdk.core.auth.ext_app_authenticator.ExtensionAppAuthenticator` instance.
        """
        app_config = self._config.app
        if app_config.is_rsa_configuration_valid():
            return ExtensionAppAuthenticatorRsa(
                AuthenticationApi(self._api_client_factory.get_login_client()),
                PodApi(self._api_client_factory.get_pod_client()),
                app_config.app_id,
                app_config.private_key,
                self._config.retry
            )
        if app_config.is_certificate_configuration_valid():
            return ExtensionAppAuthenticatorCert(
                CertificateAuthenticationApi(self._api_client_factory.get_app_session_auth_client()),
                CertificatePodApi(self._api_client_factory.get_app_session_auth_client()),
                app_config.app_id,
                self._config.retry
            )
        raise AuthInitializationError("Application under 'app' field should be configured with a private key path or "
                                      "content in order to authenticate extension app.")



class BdkServerConfig:
    """Base class for server and client configurations"""
    DEFAULT_SCHEME: str = "https"
    DEFAULT_HTTPS_PORT: int = 443

    def __init__(self, scheme=None, port=None, context="", host=None, proxy=None, default_headers=None):
        self.scheme = scheme if scheme is not None else self.DEFAULT_SCHEME
        self.port = port if port is not None else self.DEFAULT_HTTPS_PORT
        self.context = context
        self.host = host
        self.proxy = BdkProxyConfig(**proxy) if proxy is not None else None
        self.default_headers = default_headers

    def get_base_path(self) -> str:
        """Constructs the base path of the current config

        :return: scheme://host:port + formatted_context
        :rtype: str
        """
        return f"{self.scheme}://{self.host}{self.get_port_as_string()}{self.get_formatted_context()}"

    def get_formatted_context(self) -> str:
        """Formats the context field

        :return: "/" + context if the context is not empty
        :rtype: str
        """
        local_context = self.context
        if local_context is None or not isinstance(local_context, str):
            return ""
        if local_context and local_context[0] != "/":
            return "/" + local_context
        if local_context and local_context.endswith("/"):
            return local_context[0:-1]
        return local_context

    def get_port_as_string(self) -> str:
        """

        :return: the port information to be appended to the built URL
        """
        return ":" + str(self.port) if self.port else ""


class BdkProxyConfig:
    """Class to configure a proxy with a host, port and optional proxy credentials"""

    def __init__(self, host, port, username=None, password=None):
        """

        :param host: host of the proxy (mandatory)
        :param port: port of the proxy (mandatory)
        :param username: username for proxy basic authentication (optional)
        :param password: password for proxy basic authentication (optional, must be not None if username specified)
        """
        self.host = host
        self.port = port
        self.username = username
        self.password = password

    def get_url(self):
        """Builds the proxy URL.

        :return: the URL of the http proxy to target
        """
        return f"http://{self.host}:{self.port}"

    def are_credentials_defined(self):
        """Check if proxy credentials were set

        :return: True if username and password set
        """
        return self.username and self.password is not None

    def get_credentials(self):
        """Builds the credentials information to pass to the proxy-authorization header before base64 encoding.

        :return: username + ":" + password
        """
        return f"{self.username}:{self.password}"



"""Module containing session handle classes.

"""
import logging

from symphony.bdk.core.auth.exception import AuthInitializationError

logger = logging.getLogger(__name__)


class AuthSession:
    """RSA Authentication session handle to get session and key manager tokens from.
    It uses a BotAuthenticator to actually retrieve the tokens when needed.
    """

    def __init__(self, authenticator):
        """

        :param authenticator: the :class:`symphony.bdk.core.auth.bot_authenticator.BotAuthenticator` instance to
          retrieve the tokens from.
        """
        self._session_token = None
        self._key_manager_token = None
        self._authenticator = authenticator

    async def refresh(self):
        """Trigger re-authentication to refresh the tokens.
        """
        logger.debug("Authenticate")
        self._session_token = await self._authenticator.retrieve_session_token()
        self._key_manager_token = await self._authenticator.retrieve_key_manager_token()

    @property
    async def session_token(self):
        """

        :return: the session token.
        """
        if self._session_token is None:
            self._session_token = await self._authenticator.retrieve_session_token()
        return self._session_token

    @property
    async def key_manager_token(self):
        """

        :return: the key manager token
        """
        if self._key_manager_token is None:
            self._key_manager_token = await self._authenticator.retrieve_key_manager_token()
        return self._key_manager_token

    @session_token.setter
    def session_token(self, value):
        """Sets the session token. Used for testing purposes only.

        :param value: the new session token.
        """
        self._session_token = value

    @key_manager_token.setter
    def key_manager_token(self, value):
        """Sets the key manager token. Used for testing purposes only.

        :param value: the new key manager token.
        """
        self._key_manager_token = value


class OboAuthSession(AuthSession):
    """RSA OBO Authentication session handle to get the OBO session token from.
    It uses an :class:`symphony.bdk.core.auth.obo_authenticator.OboAuthenticator` to actually retrieve the tokens when
    needed.
    """

    def __init__(self, authenticator, user_id: int = None, username: str = None):
        """At least user_id or username should be defined.

        :param authenticator: the :class:`symphony.bdk.core.auth.obo_authenticator.OboAuthenticator` instance to
          retrieve the tokens from.
        :param user_id: User Id.
        :param username: Username
        """
        super().__init__(authenticator)
        if user_id is not None and username is not None:
            raise AuthInitializationError("Username and user id for OBO authentication should not be defined at "
                                          "a same time.")
        if user_id is None and username is None:
            raise AuthInitializationError("At least username or user id should be defined for "
                                          "OBO authentication.")
        self.user_id = user_id
        self.username = username

    async def refresh(self):
        """Trigger re-authentication to refresh the OBO session token.
        """
        if self.user_id is not None:
            self._session_token = await self._authenticator.retrieve_obo_session_token_by_user_id(self.user_id)
        if self.username is not None:
            self._session_token = await self._authenticator.retrieve_obo_session_token_by_username(self.username)

    @property
    async def session_token(self):
        if self._session_token is None:
            await self.refresh()
        return self._session_token

    @property
    async def key_manager_token(self):
        """

        :return: an empty string since there is no key manager token in OBO mode.
        """
        return ""


class AppAuthSession:
    """Extension application RSA authentication handle to store the tokens. It uses a
    :class:`symphony.bdk.core.auth.ext_app_authenticator.ExtensionAppAuthenticator` to actually authenticate and
    retrieve the tokens.
    """

    def __init__(self, authenticator, app_token: str):
        """

        :param authenticator: the :class:`symphony.bdk.core.auth.ext_app_authenticator.ExtensionAppAuthenticator`
          which will actually perform the authentication
        :param app_token: the application token
        """
        self._authenticator = authenticator
        self._app_token = app_token
        self._expire_at = -1
        self._symphony_token = ""

    async def refresh(self) -> None:
        """Triggers re-authentication to refresh the tokens.

        :return: None
        """
        app_tokens = await self._authenticator.authenticate_and_retrieve_tokens(self._app_token)
        self._symphony_token = app_tokens.symphony_token
        self._app_token = app_tokens.app_token
        self._expire_at = app_tokens.expire_at

    @property
    def symphony_token(self) -> str:
        """

        :return: the Symphony token
        """
        return self._symphony_token

    @property
    def app_token(self) -> str:
        """

        :return: the application token
        """
        return self._app_token

    @property
    def expire_at(self) -> int:
        """

        :return: the Unix timestamp in milliseconds of the Symphony token expiration
        """
        return self._expire_at
